function image = make_test_image(loc1, loc2, loc3)
%MAKE_TEST_IMAGE generates a shapes image
%   pixel values lie on the interval [-1,1]
%
%   make_test_image - generates an image in which the position
%      of each shape is randomly and independently generated
%
%   make_test_image(loc1,loc2,loc3) - loc1, loc2, and loc3
%      specify the locations of the box, triangle, and +
%      respectively.  Each location must be in the range [0,1].
%  
%   Author: David Ross

%------------------------------------------------------------
% check arguments
%------------------------------------------------------------

if ~(nargin == 0 | nargin == 3)
    error('make_test_image requires either 0 or 3 arguments');
end

locations_provided = 0;
if nargin == 3
   if loc1 < 0 | loc1 > 1 | loc2 < 0 | loc2 > 1 | ...
       loc3 < 0 | loc3 > 1
       error('locations must be on the interval [0,1]');
   else
       locations_provided = 1;
   end
end


IMAGE_SIZE = 11;     % length of a side of the (square) image
OBJECT_SIZE = 3;     % the size of each object in the image
SPACING = 1;         % IMAGE_SIZE = 3 * OBJECT_SIZE + 2 * SPACING

image = zeros(IMAGE_SIZE);

%------------------------------------------------------------
% box
%------------------------------------------------------------

if locations_provided
    location = loc1 * (IMAGE_SIZE - OBJECT_SIZE) + 1;
else
    location = rand(1) * (IMAGE_SIZE - OBJECT_SIZE) + 1;
end

image(:,1) = fill_column(location, OBJECT_SIZE, IMAGE_SIZE);

% copy columns
for i = 2:OBJECT_SIZE
    image(:,i) = image(:,1);
end

%------------------------------------------------------------
% triangle
%------------------------------------------------------------

triangle_offset = OBJECT_SIZE + SPACING;

if locations_provided
    location = loc2 * (IMAGE_SIZE - OBJECT_SIZE) + 1;
else
    location = rand(1) * (IMAGE_SIZE - OBJECT_SIZE) + 1;
end

loc = location;
height = OBJECT_SIZE;

for i = 1:OBJECT_SIZE
    image(:, triangle_offset + i) = fill_column(loc, height, IMAGE_SIZE);
    loc = loc + 1;
    height = height - 1;
end


%------------------------------------------------------------
% cross
%------------------------------------------------------------

cross_offset = (OBJECT_SIZE + SPACING) * 2 ;

if mod(OBJECT_SIZE, 3) == 0
    height_side = OBJECT_SIZE / 3;
    height_centre = OBJECT_SIZE / 3;
elseif mod(OBJECT_SIZE, 3) == 1
    height_side = OBJECT_SIZE / 3;
    height_centre = OBJECT_SIZE / 3 + 1;    
else % mod(OBJECT_SIZE, 3) == 2
    height_side = OBJECT_SIZE / 3 + 1;
    height_centre = OBJECT_SIZE / 3;
end

if locations_provided
    location = loc3 * (IMAGE_SIZE - OBJECT_SIZE) + 1;
else
    location = rand(1) * (IMAGE_SIZE - OBJECT_SIZE) + 1;
end

image(:,cross_offset+1) = fill_column(location + height_side, height_side, IMAGE_SIZE);

% draw the rest of the left hand size
for i = 2:height_side
    image(:, cross_offset + i) = image(:, cross_offset + 1);
end

% draw the right hand side
for i = (height_side + height_centre + 1):OBJECT_SIZE
    image(:, cross_offset + i) = image(:, cross_offset + 1);
end

% draw the centre
centre_offset = cross_offset + height_side;
image(:, centre_offset + 1) = fill_column(location, OBJECT_SIZE, IMAGE_SIZE);

for i = 1:height_centre
    image(:, centre_offset + i) = image(:, centre_offset + 1);
end

%------------------------------------------------------------
% finally scale the pixel value to lie on [-1, 1]
%------------------------------------------------------------
image = 2*image -1;