function [lambda, mu, psi, aa, gg, mm, ff] = mcfa(X, K, J, Iterations)
% [lambda, mu, psi, aa, gg, mm, ff] = mcfa(X, K, J, Iterations)
% Multiple cause factor analysis
%
% Required Arguments:
%    X = data matrix (given as column vectors), of size D*C
%    K = # of FA models to train
%    J = # of basis vectors per FA model
%    Iterations = # of EM iterations to perform
%
% Return Values:
%    lambda = factor matrices, an array of size D*J*K
%    mu = means for each FA model, an array of size D*K
%    psi = noise variance in dimension d of FA k, as
%         an array of size D*K
%    aa = gives the prior probability of selecting FA k for
%         data dimensions d, a matrix of size D*K
%    gg = the (variational) posterior of selecting FA k for d
%    mm = the factor loadings for each data and each FA, as
%         an array of size D*C*K
%    f = vector of free energies at each iteration
%
% Author: David Ross
% $Id: mcfa.m,v 1.7 2004/11/01 22:18:45 dross Exp $

%--------------------------------------
% Check arguments
%--------------------------------------
error(nargchk(4,4,nargin));

[D C] = size(X);

if K < 1 | J < 1
    error('K and J must be at least 1');
end

if Iterations < 1
    error('number of iterations must be at least 1');
end

compute_f = (nargout >= 7);


%--------------------------------------
% Initialization
%--------------------------------------
MIN_VARIANCE = 1.3460e-04;
% MIN_VARIANCE = 1e-12;


lambda = rand(D,J,K);
mu = mean(X,2)*(rand(1,K)/10 + 0.95);
psi = (var(X')' + MIN_VARIANCE) * ones(1,K);
mm = rand(J,C,K);
ss = zeros(J,J,C); % only keep this for the current K, to save memory
gg = ones(D,K) / K;  % this could be D*K*C
g_temp = zeros(size(gg)); % a temporary used for updates to gg and psi
aa = ones(D,K) / K;
ff = zeros(Iterations,1);

% temporary variables
omega = zeros(J);
x_minus_mu = zeros(D,C);


%--------------------------------------
% Main
%--------------------------------------

for it = 1:Iterations
    
    %----------------------------------
    % EM Updates
    %----------------------------------
    for kk = 1:K
        %% Update omega
        omega = pinv(eye(J) + ... % or maybe inv instead of pinv
            lambda(:,:,kk)' * diag(gg(:,kk)./psi(:,kk)) * lambda(:,:,kk));
        
        for cc = 1:C
            %% Update mm
            mm(:,cc,kk) = omega * lambda(:,:,kk)' * ...
                ((X(:,cc) - mu(:,kk)) .* gg(:,kk) ./ psi(:,kk));
            
            %% Update ss
            ss(:,:,cc) = omega + mm(:,cc,kk)*mm(:,cc,kk)';
        end
        
        %% Update g - g_temp is an intermediary term used in the updates
        %% for both gg and psi.
        % for cc = 1:C  % @TODO use less memory
        %     x_minus_mu(:,cc) = X(:,cc) - mu(:,kk);
        % end
        x_minus_mu = X - repmat(mu(:,kk), [1 C]);
        
        g_temp(:,kk) = sum(x_minus_mu.^2, 2) ...
            + diag(lambda(:,:,kk) * sum(ss(:,:,:),3) * lambda(:,:,kk)') ...
            - 2 * sum(lambda(:,:,kk) * mm(:,:,kk) .* x_minus_mu, 2);
        % g_temp(:,kk) = sum(x_minus_mu.^2 ...
        %     - (lambda(:,:,kk)*mm(:,:,kk).* x_minus_mu) , 2);
        % There is a difference between these two versions of g_temp,
        % but there shouldn't be (ideally)...
        
        %% Update mu
        mu(:,kk) = mean(X - lambda(:,:,kk) * mm(:,:,kk), 2);
        
        %% Update lambda 
        lambda(:,:,kk) = x_minus_mu * mm(:,:,kk)' * ...
            pinv(sum(ss(:,:,:),3));
        
        %% Accumulate terms for the free energy:
        if compute_f
            ff(it) = ff(it) ...
                + 0.5*trace((mm(:,:,kk)*mm(:,:,kk)' - sum(ss(:,:,:),3))*inv(omega)) ...
                ... % + (C/2)*det(omega)^-1; %@TODO this gives confusing results
                + 0.5*trace(sum(ss(:,:,:),3));
        end
    end
    
    %% Finish udating gg
    gg = -0.5 * g_temp ./ (C*psi) + log(aa ./ sqrt(psi));
    gg = softmax(gg')';
    
    %% Update psi
    psi = g_temp ./ C;
    psi = psi + MIN_VARIANCE;
    
    %% Update aa
    aa = gg;
    aa((aa < 0.001)) = 0.001;
    aa = normalize(aa,2);
    
    
    %----------------------------------
    % Free Energy
    %----------------------------------
    if compute_f
        ff(it) = ff(it) + (C/2) * sum(sum(gg .* log(psi)));
        
        % for kk = 1:K
        %     x_minus_mu = X - repmat(mu(:,kk), [1 C]);
        % 
        %     temp = sum(x_minus_mu.^2, 2);
        %     temp = temp - 2*sum(lambda(:,:,kk) * mm(:,:,kk) ...
        %         .* x_minus_mu, 2);
        %     temp = temp + diag( ...
        %         lambda(:,:,kk) * sum(ss(:,:,:,kk),3) * lambda(:,:,kk)');
        % 
        %     ff(it) = ff(it) + 0.5 * ...
        %         sum(temp .* gg(:,kk) ./ psi(:,kk));
        % 
        % end
        
        % This should be equivalent to the above.
        ff(it) = ff(it) + 0.5 * sum(sum(g_temp .* gg ./ psi));
        
        ff(it) = ff(it) + C*sum(sum(gg .* log((gg + (gg==0)) ./ aa)));
        
        disp(sprintf('done iteration %2d, F=%g', it, ff(it)));
    else
        disp(sprintf('done iteration %2d', it));
    end
    
    % 	plot_everything; disp('hit any key to continue...'); pause;
    
end
