function plot_weights(W, image_size, G, scale)
%plot_weights(W, image_size, G, scale) plots the weights produced by MCVQ
%   In the resulting plot, each row corresponds to a VQ, 
%   and the ith plot in each row corresponds to the ith
%   component of that VQ.
%
%   If G is supplied, then it's values are used as a mask
%   when plotting each VQ's weights (i.e. we black-out the 
%   image pixels that the VQ hasn't won)
%
% Author: David Ross


%-----------------------------------------------
% Check the input.
%-----------------------------------------------
error(nargchk(1,4,nargin));

[J, K, N] = size(W);

% if no image size has been specified, assume the image
% is square
if nargin == 1 | isempty(image_size)
    img_width = sqrt(N);
    img_height = img_width;
else
    img_height = image_size(1);
    img_width = image_size(2);
end

% do we mask the weights using G?
if nargin >= 3 & ~isempty(G)
    mask_weights = 1;
    if ndims(G) == 3
        disp('plot_weights - averaging G over examples');
        G = mean(G,3);    
    end
else
    mask_weights = 0;
end

if nargin <= 4
    scale = [0 1];
end
    
    
%-----------------------------------------------
% Set the figure's properties.
%-----------------------------------------------
%set(gcf, 'NumberTitle', 'off', 'MenuBar', 'none'); 

if mask_weights
    set(gcf, 'Name', 'Masked Means');
else
    set(gcf, 'Name', 'Means');
end


%-----------------------------------------------
% Plot the weights.
%-----------------------------------------------
plot_index = 1;

% scale the weights so that they lie on [0,1]
W = W - min(W(:));
W = W / max(W(:));

% use G to mask the weights
if mask_weights
    % a hard mask - plot the weight for a VQ only if it 
    % has the largest value for the pixel
    %mask = (G == repmat(max(G')', [1 K]));
    %for j = 1:J
    %    W(j,:,:) = permute(W(j,:,:), [2 3 1]) .* mask';
    %end
    
    % a soft mask
    for j = 1:J
        W(j,:,:) = permute(W(j,:,:), [2 3 1]) .* G';
    end
    
end


for k = 1:K
    my_subplot(K, J+1, plot_index);
    plot_index = plot_index + 1;
    set(gca, 'Visible', 'off');
    text(0, 0.5, sprintf('VQ %d', k));
    
    for j = 1:J
        my_subplot(K, J+1, plot_index);
        plot_index = plot_index + 1;
        % hinton(reshape(W(j,k,:), img_height, img_width));
        imagesc(reshape(W(j,k,:), img_height, img_width), scale);
        set(gca, 'Visible', 'off');
        colormap('gray');
        axis equal tight;
    end
end
