/*
 * [r,s] = sample_posterior(x,mu,sigma,a,b,samples);
 *    Use Gibb's sampling to draw samples from the posterior of MCVQ.
 *
 * Required input:
 *    x = D*1
 *    mu = D*(J*K) -> index j moves faster
 *    sigma = D*(J*K) -> index j moves faster
 *    a = K*D
 *    b = J*K, or a vector of length (J*K), where j index moves faster
 *    samples = 1*1
 *
 *  Return values:
 *     r = D*samples
 *     s = K*samples
 */

#include "mex.h"
#include <math.h>
#include <stdlib.h>

/* #define PRINT_DEBUG */

void sample_posterior(const double *x, const double *mu, 
   const double *sigma, const double *a, const double *b, 
   const int samples, const int D, const int J, const int K, 
   int *r, int *s);
void prob_of_sk(const double *x, const double *mu, const double *sigma, 
   const int *r, int *s, const int k, const int D, const int J, 
   const int K, double *result);
void softmax(double *x, int lenth);
double sum(double *x, int length);
int sample_discrete(const double *prob, const int length);
double matlab_rand();
void printit(int *x, int length, const char *name);
void printit_d(double *x, int length, const char *name);


/*
 * Take care of interfacing with Matlab
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
   /* input */
   double *x, *mu, *sigma, *a, *b;
   int samples;

   /* output */
   int *r, *s;
   double *r_double, *s_double;

   /* sizes of the matrices */
   int D, K, J;

   /* for-loop counters */
   int i;

   /* For purposes of debugging, print the memory addresses of each
      of the subfunctions.  If it segfaults, you can deduce what
      subfunction causes the problem. */
   if (nlhs == 0 && nrhs == 0) {
      printf("location of sample_posterior %lx\n", (long int) sample_posterior);
      printf("location of prob_of_sk %lx\n", (long int) prob_of_sk);
      printf("location of softmax %lx\n", (long int) softmax);
      printf("location of sum %lx\n", (long int) sum);
      printf("location of sample_discrete %lx\n", (long int) sample_discrete);
      printf("location of printit %lx\n", (long int) printit);
      printf("location of printit_d %lx\n", (long int) printit_d);
      return;
   }

   /* check for the proper number of arguments */
   if (nrhs != 6) {
      mexErrMsgTxt("wrong number of input arguments");
   }
   if (nlhs != 2) {
      mexErrMsgTxt("wrong number of output arguments");
   }

   D = mxGetM(prhs[0]);
   K = mxGetM(prhs[3])*mxGetN(prhs[3])/D;
   J = mxGetM(prhs[4])*mxGetN(prhs[4])/K;

   x = mxGetPr(prhs[0]);
   mu = mxGetPr(prhs[1]);
   sigma = mxGetPr(prhs[2]);
   a = mxGetPr(prhs[3]);
   b = mxGetPr(prhs[4]);
   samples = mxGetScalar(prhs[5]);


   /* check all the sizes matrices */
   if (!mxIsDouble(prhs[0]) || mxGetN(prhs[0]) != 1) {
      mexErrMsgTxt("x must be a column vector of doubles");
   }

   if (!mxIsDouble(prhs[1]) || mxGetM(prhs[1]) != D || 
       mxGetN(prhs[1]) != J*K) {
      mexErrMsgTxt("mu must be a double matrix of size D*JK");
   }

   if (!mxIsDouble(prhs[2]) || mxGetM(prhs[2]) != D || 
       mxGetN(prhs[2]) != J*K) {
      mexErrMsgTxt("sigma must be a double matrix of size D*JK");
   }

   if (!mxIsDouble(prhs[3]) || mxGetN(prhs[3]) != D) {
      mexErrMsgTxt("a must be a double matrix of size K*D");
   }

   if (!mxIsDouble(prhs[4]) || 
       (mxGetN(prhs[4]) != K && (mxGetN(prhs[4])*mxGetM(prhs[4]) != (J*K)))) {
      mexErrMsgTxt("b must be a double matrix of size J*K (or JK*1)");
   }

   if (!mxIsDouble(prhs[5]) || mxGetM(prhs[5]) != 1 ||
       mxGetN(prhs[5]) != 1) {
      mexErrMsgTxt("samples must be a scalar");
   }
   

   /* allocate the output vars */
   plhs[0] = mxCreateDoubleMatrix(D, samples, mxREAL);
   r_double = mxGetPr(plhs[0]);
   plhs[1] = mxCreateDoubleMatrix(K, samples, mxREAL);
   s_double = mxGetPr(plhs[1]);
   r = mxMalloc(D*samples*sizeof(*r));
   s = mxMalloc(K*samples*sizeof(*s));


   sample_posterior(x, mu, sigma, a, b, samples, D, J, K, r, s);

   for(i=0; i < D*samples; i++) {
      r_double[i] = ((double) r[i]) + 1.0;
   }

   for(i=0; i < K*samples; i++) {
      s_double[i] = ((double) s[i]) + 1.0;
   }
}


/*
 * Draw the samples, as described above.
 */
void sample_posterior(const double *x, const double *mu, const double *sigma,
		      const double *a, const double *b, const int samples, 
		      const int D, const int J, const int K,
		      int *r, int *s) {

   int d, k, j, n; /* for loop counters */

   double temp; /* for storing intermediate result */
   int index;

   double *p, *q; /*distributions @TODO */
   p = mxMalloc(J*sizeof(double));
   q = mxMalloc(K*sizeof(double));

   /* initialize r and s */
   for(d=0; d < D; d++) {
      r[d] = sample_discrete(a+K*d, K);
   }
   printit(r, D, "r_mex");

   for(k=0; k < K; k++) {
      s[k] = sample_discrete(b+J*k, J);
   }
   printit(s, K, "s_mex");

   for(n=0; n < samples; n++) {


      /*--------------------------------------------------*/
      /* sample an s */
      /*--------------------------------------------------*/
      for(k=0; k < K; k++) {
	 /* compute the prob of the joint probability when s_k == j */
	 prob_of_sk(x, mu, sigma, r+D*n, s+K*n, k, D, J, K, p);

	 printit_d(p, J, "p");

	 /* add the log prior @TODO don't repeat the log */
	 for(j=0; j < J; j++) {
	    p[j] += log(b[J*k+j]);
	 }

	 /* normalize p and sample a value */
	 softmax(p, J);
	 s[K*n + k] = sample_discrete(p, J);
      }

      printit(s+K*n, K, "sampl");

      /*--------------------------------------------------*/
      /* sample an r */
      /*--------------------------------------------------*/
      for(d=0; d < D; d++) {
	 for(k=0; k < K; k++) {
	    index = D*(J*k + s[k]) + d;
	    temp = (x[d] - mu[index]) / sigma[index];
	    q[k] = -0.5*temp*temp -log(sigma[index]);
	 }

	 /* add the log prior */
	 for(k=0; k < K; k++) {
	    q[k] += log(a[K*d+k]);
	 }

	 /* normalize q and sample a value */
	 softmax(q, K);
	 r[D*n + d] = sample_discrete(q, K);
      }

      
      /*--------------------------------------------------*/
      /* If this isn't the final sample, sample n to */
      /* n+1 in r and s.  */
      /*--------------------------------------------------*/
      if (n+1 < samples) {
	 for(d=0; d < D; d++) {
	    r[D*(n+1)+d] = r[D*n+d];
	 }
	 for(k=0; k < K; k++) {
	    s[K*(n+1)+k] = s[K*n+k];
	 }
      }
   }

   /* clean up */
   mxFree(p);
   mxFree(q);
}


void prob_of_sk(const double *x, const double *mu, const double *sigma, 
	    const int *r, int *s, const int k, 
	    const int D, const int J, const int K, 
	    double *result) {

   int d, j; /* for-loop counters */
   int index;
   double *p = NULL;
   double temp;

   p = (double*) mxMalloc(D * sizeof(*p));

   for(j=0; j < J; j++) {
      s[k] = j;
      for(d=0; d < D; d++) {
	 /* In the first pass (j==0), update everything.
	    In later passes, only update p[d] if dimension d chooses
	    VQ k (i.e. r[d] == k). */
	 if (j == 0 || r[d] == k) {
	    index = d + D*(J*r[d] + s[r[d]]);
	    temp = (x[d] - mu[index]) / sigma[index];
	    p[d] = -0.5*temp*temp - log(sigma[index]);
	    p[d] -= log(2*M_PI) / 2.0; /* @TODO this isn't necessary */
	 }
      }
      result[j] = sum(p,D);
   }

   mxFree(p);
}


/* 
 * Replace 'x' with it's softmax, where 'length' is the # of elements.
 */
void softmax(double *x, int length) {
   int i;
   double offset, sum;

   offset = x[0];
   for(i=1; i < length; i++) {
      if (x[i] > offset) {
	 offset = x[i];
      }
   }

   offset -= log(DBL_MAX)/2;

   sum = 0.0;
   for(i=0; i < length; i++) {
      x[i] = exp(x[i] - offset);
      sum += x[i];
   }
   
   for(i=0; i < length; i++) {
      x[i] /= sum;
   }
}


/*
 * Return the sum of the first 'length' elements of 'x'
 */
double sum(double *x, int length) {
   int i;
   double result = 0.0;

   for(i=0; i < length; i++) {
      result += x[i];
   }

   return result;
}


/*
 * Draw a sample from a discrete probability distribution,
 * in the range [0,length-1].
 * PRE: 'prob' is actually a probability distribution.
 */
int sample_discrete(const double *prob, const int length) {
    int i=0;
    double urand = 0.0;
    double *cumsum = NULL;
    
    cumsum = mxMalloc(sizeof(double)*length);
    cumsum[0] = prob[0];
    for(i=1; i < length; i++) {
        cumsum[i] = cumsum[i-1] + prob[i];
    }
    
    urand = ((double)rand()) / RAND_MAX;
    /*urand = matlab_rand();*/
    for(i=0; i < length && urand >= cumsum[i]; i++);
    
    mxFree(cumsum);

    return i;
}


/*
 * Get a random double by using Matlab's builtin rand function.
 */
double matlab_rand() {
   mxArray *lhs[1];
   double result;
   mexCallMATLAB(1, lhs, 0, NULL, "rand");
   result = mxGetScalar(lhs[0]);
   mxDestroyArray(lhs[0]);
   return result;
}


/*
 * Print out an integer array of the specified length with specified
 * name, in a format suitable to be used as Matlab input. 
 * Note that this only happens if PRINT_DEBUG is defined.
 */
void printit(int *x, int length, const char *name) {
#ifdef PRINT_DEBUG
   int i;
   printf("%s = [", name);
   for(i=0; i < length; i++) {
      printf("%d,", x[i]);
   }
   printf("]';\n");
#endif
}


/*
 * Print out an double array of the specified length with specified
 * name, in a format suitable to be used as Matlab input. 
 * Note that this only happens if PRINT_DEBUG is defined.
 */
void printit_d(double *x, int length, const char *name) {
#ifdef PRINT_DEBUG
   int i;
   printf("%s = [", name);
   for(i=0; i < length; i++) {
      printf("%g,", x[i]);
   }
   printf("]';\n");
#endif
}
