function [r,s] = sample_posterior(x, mu, sigma, a, b, samples)
% [r,s] = sample_posterior(x, mu, sigma, a, b, samples)
% draw n samples from P(R,S|x,mu,sigma,a,b) using Gibbs sampling
%
% Required Arguments:
%    x - N*1
%    mu - N*(J*K)
%    sigma - N*(J*K)
%    a - K*N
%    b - (J*K)*1
%    samples - integer number of samples to draw
%
% Return Values:
%    r - N*samples
%    s - N*samples
%
% Author: David Ross
% $Id: sample_posterior.m,v 1.2 2006/11/09 22:45:11 dross Exp $


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error(nargchk(5,6,nargin));
if nargin < 6
    samples = 1;
end

N = size(mu,1);
K = prod(size(a)) / N; %K = size(a,2);
J = size(b,1) / K;

if size(a,1) ~= N
    a = a';
end

%a = reshape(a',[N K]);
b = reshape(b,[J K]);


if any(size(x) ~= [N 1])
    error('size of x doesn''t match size of mu');
end

if any(size(sigma) ~= size(mu))
    error('size of sigma doesn''t match size of mu');
end

if any(size(a) ~= [N K])
    error('size of a doesn''t match size of mu');
end

% if any(size(b) ~= [J K]) % @TODO check this
%     error('size of b doesn''t match size of mu');
% end

if prod(size(samples)) > 1 | samples < 1
    error('samples must be a positive scalar');
end

% %% appropriately reshape the input
% % we want mu and sigma to be [N J*K]
% mu = reshape(ipermute(mu, [2 3 1]), [N J*K]);
% sigma = reshape(ipermute(sigma, [2 3 1]), [N J*K]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

r = zeros(N,samples);
s = zeros(K,samples);

% draw an inital (r,s) from their priors
% sample an R from a, and an S from b
r(:,1) = sample_discrete2(a',1)';
s(:,1) = sample_discrete2(b,1)';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gibb's sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for qq = 1:samples

    p_nsm = [];
    %% sample an s
    for k = 1:K
        % p(j) = log of joint probability, when s_{k} = j
        p = zeros(J,1);        
		% p = sample_posterior_helper(x, mu, sigma, ...
		%     r(:,qq), s(:,qq), k);
		for j = 1:J
		    % calculate joint probability, with s_{k} = j
		    % P(s_k=j, s_(not k) as before, R, x)
		    s(k,qq) = j;
		    indicies = s(r(:,qq),qq) + J*(r(:,qq)-1);
		    this_mu = select_from_rows(mu,indicies);
		    this_sigma = select_from_rows(sigma,indicies);
		    p(j) = gaussian_prob(x, this_mu, this_sigma.^2,1);
			% p(j) = loggprob(x, this_mu, this_sigma);
		end
        % normalize p (note: p are logprobs)
        p = softmax(p + log(b(:,k)));
        s(k,qq) = sample_discrete(p,1);
    end

    
    %% sample an r
    % compute prior probability of s, P(s) -> UNECESSARY
    % prob_s = prod(select_from_rows(b',s(:,qq))); UNECESSARY

    % given s, compute the restricted mu and sigma
    this_mu = mu(:,s(:,qq)' + J*(0:(K-1)));
    this_sigma = sigma(:,s(:,qq)' + J*(0:(K-1)));

    % calculate probabilities, normalize, then sample
    % p = normpdf(x*ones(1,K), this_mu, this_sigma) .* a * prob_s; UNECESSARY
	% p = normpdf(x*ones(1,K), this_mu, this_sigma) .* a;
	% p = normalize(p,2);
    
    % --> do it in logs instead
    p = -0.5* ((x*ones(1,K)-this_mu)./this_sigma).^2 - log(this_sigma);
    % p = p - 0.5*log(2*pi);
    p = softmax((p + log(a))')';
    
    r(:,qq) = sample_discrete2(p',1)';
    
    % copy sample qq into slot qq+1, unless we're done
    if qq < samples
        s(:,qq+1) = s(:,qq);
        r(:,qq+1) = r(:,qq);
    end
end
